/**************************************************************************
*  Copyright (c) 2015-2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  19.09.2015  mifi  First Version, tested with a STM3240G-EVAL board.
*  07.11.2015  mifi  Corrected project for CrossWorks for ARM v2.3.5.
*  06.02.2016  mifi  Tested with TinyCTS v0.10.
*  12.03.2016  mifi  Tested with TinyCTS v0.11.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <string.h>

#include "tal.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define LED_LD1_ON()    { GPIOG->BSRRL = (1<<6); }
#define LED_LD1_OFF()   { GPIOG->BSRRH = (1<<6); } 

#define LED_LD2_ON()    { GPIOG->BSRRL = (1<<8); }
#define LED_LD2_OFF()   { GPIOG->BSRRH = (1<<8); } 

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/
/* 
 * Some TASK variables like stack and task control block.
 */
static OS_STACK (LEDStack,   TASK_LED_STK_SIZE);
static OS_STACK (StartStack, TASK_START_STK_SIZE);
static OS_STACK (Float1Stack, TASK_FLOAT1_STK_SIZE);
static OS_STACK (Float2Stack, TASK_FLOAT2_STK_SIZE);

static os_tcb_t TCBLed;
static os_tcb_t TCBStartTask;
static os_tcb_t TCBFloat1;
static os_tcb_t TCBFloat2;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  NewSysTick_Config                                                    */
/*                                                                       */
/*  Based on the "core_cm4.h" version, but an AHB clock divided by 8 is  */
/*  used for the SysTick clock source.                                   */
/*                                                                       */
/*  The function initializes the System Timer and its interrupt, and     */
/*  starts the System Tick Timer. Counter is in free running mode to     */
/*  generate periodic interrupts.                                        */
/*                                                                       */
/*  In    : ticks                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static uint32_t NewSysTick_Config (uint32_t ticks)
{
   /*
    * AHB clock divided by 8 is used for the SysTick clock source.
    */
   ticks = ticks / 8;

   /* Reload value impossible */
   if ((ticks - 1) > SysTick_LOAD_RELOAD_Msk)  return (1);

   /* Set reload register */
   SysTick->LOAD = ticks - 1;
  
   /* Set Priority for Systick Interrupt */
   NVIC_SetPriority(SysTick_IRQn, 0);
  
   /* Load the SysTick Counter Value */
   SysTick->VAL = 0;
   
   /* Enable SysTick IRQ and SysTick Timer */
   SysTick->CTRL = SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;
   
   return(ticks);
} /* NewSysTick_Config */

/***************************************************************************/
/*  LEDInit                                                                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LEDInit (void)
{
   /* Turn on GPIOC, GPIOG and GPIOI clock */
   RCC->AHB1ENR = RCC_AHB1ENR_GPIOCEN | RCC_AHB1ENR_GPIOGEN | RCC_AHB1ENR_GPIOIEN;
   
   /* Switch off all LEDs */
   GPIOG->BSRRH = (1<<6);  /* LD1, green */
   GPIOG->BSRRH = (1<<8);  /* LD2, yellow */
   GPIOI->BSRRH = (1<<9);  /* LD3 */
   GPIOC->BSRRH = (1<<7);  /* LD4 */

   /* Set PG6 to output */
   GPIOG->MODER &= ~GPIO_MODER_MODER6;
   GPIOG->MODER |= GPIO_MODER_MODER6_0;
   
   /* Set PG8 to output */
   GPIOG->MODER &= ~GPIO_MODER_MODER8;
   GPIOG->MODER |= GPIO_MODER_MODER8_0;

   /* Set PI9 to output */
   GPIOI->MODER &= ~GPIO_MODER_MODER9;
   GPIOI->MODER |= GPIO_MODER_MODER9_0;

   /* Set PC7 to output */
   GPIOC->MODER &= ~GPIO_MODER_MODER7;
   GPIOC->MODER |= GPIO_MODER_MODER7_0;

} /* LEDInit */

/***************************************************************************/
/*  HardwareInit                                                           */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void HardwareInit (void)
{
   /* Update clock info */
   SystemCoreClockUpdate(); 

   /* Init SysTick */   
   NewSysTick_Config(SystemCoreClock / OS_TICKS_PER_SECOND);

   /* Configure Priority Grouping */
   NVIC_SetPriorityGrouping(7);
   
   /* System configuration controller clock enable */   
   RCC->APB2ENR |= RCC_APB2ENR_SYSCFGEN;
   
   /* Init LED */
   LEDInit();
} /* HardwareInit */

/*************************************************************************/
/*  Float1Task                                                           */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void Float1Task (void *p)
{
   float fa = 0.1f;
   
   (void)p;
   
   while(1)
   {
      fa = fa + 0.1f;
      
      OSTimeDly(TASK_FLOAT1_DELAY_MS);
   }
} /* Float1Task */

/*************************************************************************/
/*  Float2Task                                                           */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void Float2Task (void *p)
{
   float fa = 0.7f;
   
   (void)p;
   
   while(1)
   {
      fa = fa + 0.7f;
      
      OSTimeDly(TASK_FLOAT2_DELAY_MS);
   }
} /* Float2Task */

/*************************************************************************/
/*  LEDTask                                                              */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void LEDTask (void *p)
{
   (void)p;
   
   while(1)
   {
      LED_LD1_ON();
      OSTimeDly(TASK_LED_DELAY_MS);
      LED_LD1_OFF();
      OSTimeDly(TASK_LED_DELAY_MS);
   }
} /* LEDTask */

/*************************************************************************/
/*  StartTask                                                            */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void StartTask (void *p)
{
   (void)p;
   
   /*
    * The StartTask will be used to start all other tasks in the system.
    * At the end the priority will be set to "IDLE" priority.
    */
    
   /*
    * Start the System ticker
    */
   OSSysTickStart();    

   /* Create the LED task */
   OSTaskCreate(&TCBLed, LEDTask, NULL, TASK_LED_PRIORITY,
                LEDStack, sizeof(LEDStack), 
                "LEDTask");

   /* Create the FLOAT1 task */
   OSTaskCreate(&TCBFloat1, Float1Task, NULL, TASK_FLOAT1_PRIORITY,
                Float1Stack, sizeof(Float1Stack), 
                "Float1Task");
   
   /* Create the FLOAT2 task */
   OSTaskCreate(&TCBFloat2, Float2Task, NULL, TASK_FLOAT2_PRIORITY,
                Float2Stack, sizeof(Float2Stack), 
                "Float2Task");
                
   OSTaskChangePriority(TASK_START_PRIORITY_IDLE);

   while (1)
   {
      LED_LD2_ON();
      OSTimeDly(TASK_START_DELAY_MS);
      LED_LD2_OFF();
      OSTimeDly(TASK_START_DELAY_MS);
   }
} /* StartTask */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
int main (void)
{
   /*
    * Before the application which is using the TinyCTS can be started, 
    * the hardware of the target must be initialized. The CPU is still
    * initialized by the CrossWorks for ARM startup. But the timer tick
    * must be init too. Even the GPIO can be init here.
    */
   HardwareInit();
   
   /* Init the TinyCTS */   
   OSInit();
   
   /* 
    * Create the StartTask.
    * The StartTask is the one and only task which 
    * will start all other tasks of the system.
    */
   OSTaskCreate(&TCBStartTask, StartTask, NULL, TASK_START_PRIORITY,
                StartStack, sizeof(StartStack), 
                "StartTask");
   
   /* 
    * OSStart must be the last function here.
    *  
    * Fasten your seatbelt, engine will be started...
    */        
   OSStart();
   
   /*
    * This return here make no sense.
    * But to prevent the compiler warning:
    *    "return type of 'main' is not 'int'
    * We use an int as return :-)
    */ 
   return(0); /*lint !e527*/
} /* main */

/*************************************************************************/
/*  SysTick_Handler                                                      */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void SysTick_Handler (void)
{
   TAL_CPU_IRQ_ENTER();
   
   OSTimerCallback();
   
   TAL_CPU_IRQ_EXIT();
} /* SysTick_Handler */

/*** EOF ***/
